# Hashy

Hashy is small [node.js](http://nodejs.org/) library which aims to do
passwords hashing *[the correct
way](https://wiki.php.net/rfc/password_hash)*.

It has been heavily inspired by the new [PHP password hashing
API](http://www.php.net/manual/en/book.password.php) but, following
the node.js philosophy, hashing is done asynchronously.

Furthermore, to make the interfaces as easy to use as possible, async
functions do not rely on callbacks but return
[Q](https://github.com/kriskowal/q)
[promises](https://github.com/kriskowal/q).

## Why a new library?

The other ones I found were too complicated and/or were missing
important features.

The main missing feature is the `needRehash()` function: cryptography
is a fast-moving science and algorithms can quickly become obsolete or
their parameters needs to be adjusted to compensate the performance
increase of recent computers (e.g. [bcrypt cost
factor](http://phpmaster.com/why-you-should-use-bcrypt-to-hash-stored-passwords/)).

This is exactly what this function is for: checking whether a hash
uses the correct algorithm (and options) to see if we need to compute
a new hash for this password.

## How to use it?

First, you may take a look at [the available example](https://github.com/julien-f/nodejs-hashy/blob/master/examples/basic.js).

### Creating a hash

```js
hashy.hash(password)
	.then(function (hash) {
		console.log('generated hash: ', hash);
	})
	.fail(function (err) {
		console.error(err);
	})
	.done();
```

`hash()` handles additionaly two optional parameters:

1. `algo`: which algorithm to use, it defaults to `hashy.DEFAULT`
which currently equals to `hash.BCRYPT`;
2. `options`: additional options for the current algorithm, for bcrypt
it defaults to `{cost: 10}.`.


### Checking a password against a hash

```js
hashy.verify(password, hash)
	.then(function (success) {
		if (success)
		{
			console.log('you are now authenticated!');
		}
		else
		{
			console.warn('invalid password!');
		}
	})
	.fail(function (err) {
		console.error(err);
	})
	.done();
```

### Getting information about a hash

```js
var info = hashy.getInfo(hash);
```

### Checking whether a hash is up to date

As I said [earlier](#why-a-new-library), we must be able to check
whether the hash is up to date, i.e. if it has been generated by the
last algorithm available with the last set of options.

```js
if (hashy.needsRehash(hash))
{
	// Rehash.
}
```

It handles the optional `algo` and `options` parameters like
[`hash()`](#creating-a-hash).

## License

Hashy is released under the [MIT
license](https://en.wikipedia.org/wiki/MIT_License).
